@@ -634,6 +634,9 @@ def input_split_file(absolute: bool, output_dir: str, output_samples_file: str,
               help='Skip samples that already exist in the index. Attempts to automatically detect sample names'
                    ' from file names if necessary.',
               default=False)
+@click.option('--snpeff-reference-check/--no-snpeff-reference-check',
+              help='Enable or disable checking of CDS and Protein when building snpeff reference genome database',
+              default=True)
 @click.argument('genomes', type=click.Path(exists=True), nargs=-1)
 def analysis(ctx, reference_file: str, load_data: bool, index_unknown: bool, clean: bool, build_tree: bool,
              align_type: str,
@@ -646,6 +649,7 @@ def analysis(ctx, reference_file: str, load_data: bool, index_unknown: bool, cle
              input_genomes_file: str, input_structured_genomes_file: str,
              check_files_exist: bool,
              skip_existing_samples: bool,
+             snpeff_reference_check: bool,
              genomes: List[str]):
     project = get_project_exit_on_error(ctx)
     data_index_connection = project.create_connection()
@@ -691,7 +695,8 @@ def analysis(ctx, reference_file: str, load_data: bool, index_unknown: bool, cle
                                                   kmer_scaled=kmer_scaled,
                                                   snakemake_input_batch_size=batch_size,
                                                   reads_mincov=reads_mincov,
-                                                  reads_minqual=reads_minqual)
+                                                  reads_minqual=reads_minqual,
+                                                  snpeff_no_check=not snpeff_reference_check)
 
     if sample_files is None:
         logger.info(f'Automatically structuring {len(genome_paths)} input files into assemblies/reads')
